﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyShip
{
    internal class DrawningWarmlyShip : DrawningShip
    {
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес коробля</param>
        /// <param name="bodyColor">Цвет коробля</param>
        /// <param name="dopColor">Дополнительный цвет</param>
        /// <param name="pipes">Признак наличия труб</param>
        /// <param name="fuelCompartment">Признак наличия топливного отсека</param>
        public DrawningWarmlyShip(int speed, float weight, Color bodyColor, Color dopColor, bool pipes, bool fuelCompartment) :
            base(speed, weight, bodyColor, 170, 95)
        {
            Ship = new EntityWarmlyShip(speed, weight, bodyColor, dopColor, pipes, fuelCompartment);
        }
        public override void DrawTransport(Graphics g)
        {
            if (Ship is not EntityWarmlyShip warmlyShip)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush dopBrush = new SolidBrush(warmlyShip.DopColor);

            if (warmlyShip.Pipes)
            {
                g.FillRectangle(dopBrush, _startPosX + 60, _startPosY + 10, 20, 26);
                g.DrawRectangle(pen, _startPosX + 60, _startPosY + 10, 20, 26);
                g.FillRectangle(dopBrush, _startPosX + 90, _startPosY + 5, 20, 31);
                g.DrawRectangle(pen, _startPosX + 90, _startPosY + 5, 20, 31);
                g.FillRectangle(dopBrush, _startPosX + 120, _startPosY + 1, 20, 35);
                g.DrawRectangle(pen, _startPosX + 120, _startPosY + 1, 20, 35);

            }

            _startPosY += 30;
            base.DrawTransport(g);
            _startPosY -= 30;

            if (warmlyShip.FuelCompartment)
            {
                g.FillRectangle(dopBrush, _startPosX + 135, _startPosY + 70, 10, 5);
                g.DrawRectangle(pen, _startPosX + 135, _startPosY + 70, 10, 5);
            }
        }
        public void SetDopColor(Color color)
        {
            Ship = Ship as EntityWarmlyShip;
            if (Ship is not null)
            {
                Ship = new EntityWarmlyShip(Ship.Speed, Ship.Weight, Ship.BodyColor, color, (Ship as EntityWarmlyShip).Pipes, (Ship as EntityWarmlyShip).FuelCompartment);
            }
        }
    }
}
