﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WarmlyShip
{
    public partial class FormShipConfig : Form
    {
        private event Action<DrawningShip> EventAddShip;

        DrawningShip _ship = null;
        public FormShipConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (sender, e) => Close();
        }
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void DrawShip()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _ship?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _ship?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        public void AddEvent(Action<DrawningShip> ev)
        {
            if (EventAddShip == null)
            {
                EventAddShip = new(ev);
            }
            else
            {
                EventAddShip += ev;
            }
        }
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _ship = new DrawningShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _ship = new DrawningWarmlyShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxPipes.Checked, checkBoxFuelCompartment.Checked);
                    break;
            }
            DrawShip();
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddShip?.Invoke(_ship);
            Close();
        }

        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            _ship.SetBaseColor((Color)e.Data.GetData(typeof(Color)));
            DrawShip();
        }

        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_ship is DrawningWarmlyShip)
            {
                var ship = _ship as DrawningWarmlyShip;
                ship.SetDopColor((Color)e.Data.GetData(typeof(Color)));
            }
            DrawShip();
        }
    }
}
