﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyShip
{
    internal class SetShipsGeneric<T>
        where T : class
    {
		/// <summary>
		/// Список объектов, которые храним
		/// </summary>
		private readonly List<T> _places;
		/// <summary>
		/// Количество объектов в списке
		/// </summary>
		public int Count => _places.Count;

		private readonly int _maxCount;
		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="count"></param>
		public SetShipsGeneric(int count)
		{
			_maxCount = count;
			_places = new List<T>();
		}
		/// <summary>
		/// Добавление объекта в набор
		/// </summary>
		/// <param name="ship">Добавляемый корабль</param>
		/// <returns></returns>
		public int Insert(T ship)
		{
			return Insert(ship, 0);
		}

		private bool isCorrectPosition(int position)
		{
			return 0 <= position && position < _maxCount;
		}
		/// <summary>
		/// Добавление объекта в набор на конкретную позицию
		/// </summary>
		/// <param name="ship">Добавляемый корабль</param>
		/// <param name="position">Позиция</param>
		/// <returns></returns>
		public int Insert(T airplane, int position)
		{
			if (Count == _maxCount)
				throw new StorageOverflowException(_maxCount);
			if (!isCorrectPosition(position))
			{
				return -1;
			}
			_places.Insert(position, airplane);
			return position;
		}
		/// <summary>
		/// Удаление объекта из набора с конкретной позиции
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public T Remove(int position)
		{
			if (!isCorrectPosition(position))
				return null;
			var result = this[position];
			if (result == null)
				throw new ShipNotFoundException(position);
			_places.RemoveAt(position);
			return result;
		}
		/// <summary>
		/// Получение объекта из набора по позиции
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public T this[int position]
		{
			get
			{
				return isCorrectPosition(position) && position < Count ? _places[position] : null;
			}
			set
			{
				Insert(value, position);
			}
		}
		/// <summary>
		/// Проход по набору до первого пустого
		/// </summary>
		/// <returns></returns>
		public IEnumerable<T> GetShips()
		{
			foreach (var ship in _places)
			{
				if (ship != null)
				{
					yield return ship;
				}
				else
				{
					yield break;
				}
			}
		}
	}
}
